<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    Woocommerce_Wallet_System
 * @subpackage Woocommerce_Wallet_System/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Woocommerce_Wallet_System
 * @subpackage Woocommerce_Wallet_System/includes
 * @author     WP Swings <webmaster@wpswings.com>
 */
class Woocommerce_Wallet_System {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Woocommerce_Wallet_System_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $wws_onboard    To initializsed the object of class onboard.
	 */
	protected $wws_onboard;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area,
	 * the public-facing side of the site and common side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		if ( defined( 'WOOCOMMERCE_WALLET_SYSTEM_VERSION' ) ) {

			$this->version = WOOCOMMERCE_WALLET_SYSTEM_VERSION;
		} else {

			$this->version = '1.0.3';
		}

		$this->plugin_name = 'woocommerce-wallet-system';

		$this->woocommerce_wallet_system_dependencies();
		$this->woocommerce_wallet_system_locale();
		if ( is_admin() ) {
			$this->woocommerce_wallet_system_admin_hooks();
		} else {
			$this->woocommerce_wallet_system_public_hooks();
		}
		$this->woocommerce_wallet_system_common_hooks();

		$this->woocommerce_wallet_system_api_hooks();

		// custom function for ajax.
		$this->woocommerce_wallet_system_ajax_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Woocommerce_Wallet_System_Loader. Orchestrates the hooks of the plugin.
	 * - Woocommerce_Wallet_System_i18n. Defines internationalization functionality.
	 * - Woocommerce_Wallet_System_Admin. Defines all hooks for the admin area.
	 * - Woocommerce_Wallet_System_Common. Defines all hooks for the common area.
	 * - Woocommerce_Wallet_System_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-wallet-system-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-wallet-system-i18n.php';

		if ( is_admin() ) {

			// The class responsible for defining all actions that occur in the admin area.
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woocommerce-wallet-system-admin.php';
		} else {

			// The class responsible for defining all actions that occur in the public-facing side of the site.
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-woocommerce-wallet-system-public.php';

		}

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woocommerce-wallet-system-ajax-handler.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'package/rest-api/class-woocommerce-wallet-system-rest-api.php';

		/**
		 * This class responsible for defining common functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'common/class-woocommerce-wallet-system-common.php';

		/**
		 * The class responsible for creating wallet widget for elementor.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'elementor-widget/class-elementor-wallet-widget.php';

		/**
		 * The class responsible for creating the wallet widget to show in sidebar.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wws-wallet-widget.php';

		$this->loader = new Woocommerce_Wallet_System_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Woocommerce_Wallet_System_I18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_locale() {

		$plugin_i18n = new Woocommerce_Wallet_System_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_admin_hooks() {

		$wws_plugin_admin = new Woocommerce_Wallet_System_Admin( $this->wws_get_plugin_name(), $this->wws_get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $wws_plugin_admin, 'wws_admin_enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $wws_plugin_admin, 'wws_admin_enqueue_scripts' );

		// Add settings menu for WooCommerce Wallet System.
		$this->loader->add_action( 'admin_menu', $wws_plugin_admin, 'wws_options_page' );
		$this->loader->add_action( 'admin_menu', $wws_plugin_admin, 'mwb_wws_remove_default_submenu', 50 );

		$this->loader->add_filter( 'wws_template_settings_array', $wws_plugin_admin, 'wws_admin_template_settings_page', 10 );

		$this->loader->add_filter( 'mwb_wsfw_plug_extra_tabs', $wws_plugin_admin, 'mwb_wws_plug_extra_tabs', 10, 1 );
		$this->loader->add_filter( 'mwb_wsfw_template_path', $wws_plugin_admin, 'mwb_wws_template_path', 10, 1 );
		$this->loader->add_filter( 'mwb_wsfw_show_additional_section', $wws_plugin_admin, 'mwb_wws_show_additional_section' );
		$this->loader->add_filter( 'mwb_wsfw_overview_additional_content', $wws_plugin_admin, 'mwb_wws_include_overview' );
		// license variables.
		$callname_lic         = self::$lic_callback_function;
		$callname_lic_initial = self::$lic_ini_callback_function;
		$day_count            = self::$callname_lic_initial();
		if ( self::$callname_lic() || 0 <= $day_count ) {
			$this->loader->add_action( 'mwb_wws_check_license_daily', $wws_plugin_admin, 'mwb_wws_check_license' );
		}

		if ( self::$callname_lic() || $day_count > 0 ) {
			$this->loader->add_filter( 'wsfw_general_extra_settings_array', $wws_plugin_admin, 'wws_admin_general_settings_page', 20 );
			$this->loader->add_filter( 'wsfw_check_order_meta_for_userid', $wws_plugin_admin, 'wws_check_order_meta_for_userid', 10, 2 );
			$this->loader->add_filter( 'wsfw_check_order_meta_for_recharge_reason', $wws_plugin_admin, 'wws_check_order_meta_for_recharge_reason', 10, 2 );
			$this->loader->add_filter( 'mwb_wsfw_update_wallet_to_base_price', $wws_plugin_admin, 'mwb_wws_update_wallet_to_base_price', 10, 2 );
		}

		$saved_wallet_page_id = get_option( 'mwb_wws_saved_wallet_page_id', '' );
		if ( isset( $saved_wallet_page_id ) && 'true' !== $saved_wallet_page_id ) {
			$this->loader->add_action( 'init', $wws_plugin_admin, 'mwb_wws_save_wallet_page_id', 10 );
		}

	}

	/**
	 * Register all of the hooks related to the common functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_common_hooks() {

		$wws_plugin_common = new Woocommerce_Wallet_System_Common( $this->wws_get_plugin_name(), $this->wws_get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $wws_plugin_common, 'wws_common_enqueue_styles' );

		$this->loader->add_action( 'wp_enqueue_scripts', $wws_plugin_common, 'wws_common_enqueue_scripts' );
		// license validation.
		$this->loader->add_action( 'wp_ajax_mwb_wws_validate_license_key', $wws_plugin_common, 'mwb_wws_validate_license_key' );
		// license variables.
		$callname_lic         = self::$lic_callback_function;
		$callname_lic_initial = self::$lic_ini_callback_function;
		$day_count            = self::$callname_lic_initial();
		if ( self::$callname_lic() || 0 <= $day_count ) {
			$this->loader->add_filter( 'mwb_wsfw_common_update_wallet_to_base_price', $wws_plugin_common, 'mwb_wws_common_update_wallet_to_base_price', 10, 2 );
			$enable = get_option( 'mwb_wsfw_enable', '' );
			if ( isset( $enable ) && 'on' === $enable ) {
				$this->loader->add_action( 'plugins_loaded', $wws_plugin_common, 'mwb_wsfw_wallet_shortcodes' );
				$this->loader->add_action( 'init', $wws_plugin_common, 'mwb_wsfw_save_wallet_public_shortcode' );
			}
		}
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_public_hooks() {

		$wws_plugin_public = new Woocommerce_Wallet_System_Public( $this->wws_get_plugin_name(), $this->wws_get_version() );

		// license validation.
		$callname_lic         = self::$lic_callback_function;
		$callname_lic_initial = self::$lic_ini_callback_function;
		$day_count            = self::$callname_lic_initial();
		if ( self::$callname_lic() || 0 <= $day_count ) {
			$enable = get_option( 'mwb_wsfw_enable', '' );
			if ( isset( $enable ) && 'on' === $enable ) {
				$this->loader->add_action( 'wp_enqueue_scripts', $wws_plugin_public, 'wws_public_enqueue_styles' );
				$this->loader->add_action( 'wp_enqueue_scripts', $wws_plugin_public, 'wws_public_enqueue_scripts' );
				$this->loader->add_action( 'init', $wws_plugin_public, 'mwb_wws_wallet_register_endpoint' );
				$this->loader->add_action( 'query_vars', $wws_plugin_public, 'mwb_wws_wallet_query_var' );
				$this->loader->add_action( 'woocommerce_account_mwb-wallet_endpoint', $wws_plugin_public, 'mwb_wws_display_wallet_endpoint_content', 10 );
				$this->loader->add_filter( 'template_include', $wws_plugin_public, 'mwb_wws_show_qrcode_image' );
				$this->loader->add_filter( 'the_content', $wws_plugin_public, 'content_for_my_wallet_page_wallet_transfer', 20, 1 );
				$this->loader->add_filter( 'woocommerce_add_cart_item_data', $wws_plugin_public, 'add_wallet_user_id_in_cart', 10, 2 );
				$this->loader->add_filter( 'woocommerce_get_cart_item_from_session', $wws_plugin_public, 'mwb_wws_get_cart_items_from_session', 1, 3 );
				$this->loader->add_action( 'woocommerce_add_order_item_meta', $wws_plugin_public, 'mwb_wws_add_values_to_order_item_meta', 1, 2 );
				$this->loader->add_filter( 'wsfw_min_max_value_for_wallet_recharge', $wws_plugin_public, 'min_max_value_for_wallet_recharge', 10 );
				$this->loader->add_filter( 'wsfw_check_order_meta_for_userid', $wws_plugin_public, 'wws_check_order_meta_for_userid', 1, 2 );
				$this->loader->add_filter( 'wsfw_check_order_meta_for_recharge_reason', $wws_plugin_public, 'wws_check_order_meta_for_recharge_reason', 10, 2 );
				$this->loader->add_action( 'mwb_wsfw_remove_value_from_session', $wws_plugin_public, 'mwb_wss_remove_value_from_session', 1, 1 );
				$this->loader->add_filter( 'mwb_wsfw_show_withdrawal_message', $wws_plugin_public, 'mwb_wws_show_withdrawal_message', 1 );
				$this->loader->add_filter( 'wsfw_add_invitation_link_message', $wws_plugin_public, 'mwb_wws_add_invitation_link_message', 10 );
				$this->loader->add_filter( 'mwb_wsfw_show_additional_content', $wws_plugin_public, 'mwb_wws_show_additional_content', 10, 4 );
				$this->loader->add_action( 'woocommerce_init', $wws_plugin_public, 'mwb_wws_set_woocoomerce_session', 10 );
				$this->loader->add_action( 'user_register', $wws_plugin_public, 'mwb_save_wallet_amount_after_registration', 10, 1 );
				$this->loader->add_filter( 'mwb_wsfw_show_converted_price', $wws_plugin_public, 'mwb_wws_show_converted_price', 10, 1 );
				$this->loader->add_filter( 'mwb_wsfw_convert_to_base_price', $wws_plugin_public, 'mwb_wws_convert_to_base_price', 10, 1 );
				$this->loader->add_filter( 'mwb_wsfw_get_current_currency', $wws_plugin_public, 'mwb_wws_get_current_currency', 10, 1 );

				$this->loader->add_action( 'init', $wws_plugin_public, 'mwb_wws_send_mail_to_user' );

			}
		}

	}

	/**
	 * Register all of the hooks related to the api functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_api_hooks() {

		$wws_plugin_api = new Woocommerce_Wallet_System_Rest_Api( $this->wws_get_plugin_name(), $this->wws_get_version() );

		$this->loader->add_action( 'rest_api_init', $wws_plugin_api, 'mwb_wws_add_endpoint' );

	}

	/**
	 * Register all of the hooks related to ajax
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function woocommerce_wallet_system_ajax_hooks() {

		$wsfw_plugin_ajax = new Wallet_System_AjaxHandler1();

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function wws_run() {
		$this->loader->wws_run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function wws_get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Woocommerce_Wallet_System_Loader    Orchestrates the hooks of the plugin.
	 */
	public function wws_get_loader() {
		return $this->loader;
	}


	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Woocommerce_Wallet_System_Onboard    Orchestrates the hooks of the plugin.
	 */
	public function wws_get_onboard() {
		return $this->wws_onboard;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function wws_get_version() {
		return $this->version;
	}

	/**
	 * Check license validity.
	 *
	 * @var string
	 */
	public static $lic_callback_function = 'check_lcns_validity';

	/**
	 * Check license initial days.
	 *
	 * @var string
	 */
	public static $lic_ini_callback_function = 'check_lcns_initial_days';

	/**
	 * Predefined default mwb_wws_plug tabs.
	 *
	 * @return  Array       An key=>value pair of WooCommerce Wallet System tabs.
	 */
	public function mwb_wws_plug_default_tabs() {

		$wws_default_tabs = array();

		$wws_default_tabs['woocommerce-wallet-system-general'] = array(
			'title'       => esc_html__( 'General Setting', 'woocommerce-wallet-system' ),
			'name'        => 'woocommerce-wallet-system-general',
		);
		$wws_default_tabs = apply_filters( 'mwb_wws_plugin_standard_admin_settings_tabs', $wws_default_tabs );

		$wws_default_tabs['woocommerce-wallet-system-system-status'] = array(
			'title'       => esc_html__( 'System Status', 'woocommerce-wallet-system' ),
			'name'        => 'woocommerce-wallet-system-system-status',
		);
		$wws_default_tabs['woocommerce-wallet-system-template'] = array(
			'title'       => esc_html__( 'Templates', 'woocommerce-wallet-system' ),
			'name'        => 'woocommerce-wallet-system-template',
		);
		$wws_default_tabs['woocommerce-wallet-system-overview'] = array(
			'title'       => esc_html__( 'Overview', 'woocommerce-wallet-system' ),
			'name'        => 'woocommerce-wallet-system-overview',
		);

		return $wws_default_tabs;
	}

	/**
	 * Locate and load appropriate tempate.
	 *
	 * @since   1.0.0
	 * @param string $path path file for inclusion.
	 * @param array  $params parameters to pass to the file for access.
	 */
	public function mwb_wws_plug_load_template( $path, $params = array() ) {

		$wws_file_path = WOOCOMMERCE_WALLET_SYSTEM_DIR_PATH . $path;

		if ( file_exists( $wws_file_path ) ) {

			include $wws_file_path;
		} else {

			/* translators: %s: file path */
			$wws_notice = sprintf( esc_html__( 'Unable to locate file at location "%s". Some features may not work properly in this plugin. Please contact us!', 'woocommerce-wallet-system' ), $wws_file_path );
			$this->mwb_wws_plug_admin_notice( $wws_notice, 'error' );
		}
	}

	/**

	 * Check the license.

	 * @since 1.0.0

	 * @return boolean true/false
	 */
	public static function check_lcns_validity() {
		$wws_lic_key   = get_option( 'mwb_wws_lic_key', '' );
		$wws_lic_valid = get_option( 'mwb_wws_pro_valid_license', false );
		if ( ! empty( $wws_lic_key ) && $wws_lic_valid ) {
			return true;
		} else {
			return false;
		}
	}

	/**

	 * Validate the use of features of this plugin for initial days.
	 *
	 * @since 1.0.0
	 * @return boolean true/false
	 */
	public static function check_lcns_initial_days() {

		$thirty_days  = get_option( 'mwb_wws_activated_timestamp', 0 );

		$current_time = current_time( 'timestamp' );

		$day_count = ( $thirty_days - $current_time ) / ( 24 * 60 * 60 );

		return $day_count;

	}

	/**
	 * Show admin notices.
	 *
	 * @param  string $wws_message    Message to display.
	 * @param  string $type       notice type, accepted values - error/update/update-nag.
	 * @since  1.0.0
	 */
	public static function mwb_wws_plug_admin_notice( $wws_message, $type = 'error' ) {

		$wws_classes = 'notice ';

		switch ( $type ) {

			case 'update':
				$wws_classes .= 'updated is-dismissible';
				break;

			case 'update-nag':
				$wws_classes .= 'update-nag is-dismissible';
				break;

			case 'success':
				$wws_classes .= 'notice-success is-dismissible';
				break;

			default:
				$wws_classes .= 'notice-error is-dismissible';
		}

		$wws_notice  = '<div class="' . esc_attr( $wws_classes ) . ' mwb-errorr-8">';
		$wws_notice .= '<p>' . esc_html( $wws_message ) . '</p>';
		$wws_notice .= '</div>';

		echo wp_kses_post( $wws_notice );
	}


	/**
	 * Show WordPress and server info.
	 *
	 * @return  Array $wws_system_data       returns array of all wordpress and server related information.
	 * @since  1.0.0
	 */
	public function mwb_wws_plug_system_status() {
		global $wpdb;
		$wws_system_status    = array();
		$wws_wordpress_status = array();
		$wws_system_data      = array();

		// Get the web server.
		$wws_system_status['web_server'] = isset( $_SERVER['SERVER_SOFTWARE'] ) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_SOFTWARE'] ) ) : '';

		// Get PHP version.
		$wws_system_status['php_version'] = function_exists( 'phpversion' ) ? phpversion() : __( 'N/A (phpversion function does not exist)', 'woocommerce-wallet-system' );

		// Get the server's IP address.
		$wws_system_status['server_ip'] = isset( $_SERVER['SERVER_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_ADDR'] ) ) : '';

		// Get the server's port.
		$wws_system_status['server_port'] = isset( $_SERVER['SERVER_PORT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_PORT'] ) ) : '';

		// Get the uptime.
		$wws_system_status['uptime'] = function_exists( 'exec' ) ? @exec( 'uptime -p' ) : __( 'N/A (make sure exec function is enabled)', 'woocommerce-wallet-system' );

		// Get the server path.
		$wws_system_status['server_path'] = defined( 'ABSPATH' ) ? ABSPATH : __( 'N/A (ABSPATH constant not defined)', 'woocommerce-wallet-system' );

		// Get the OS.
		$wws_system_status['os'] = function_exists( 'php_uname' ) ? php_uname( 's' ) : __( 'N/A (php_uname function does not exist)', 'woocommerce-wallet-system' );

		// Get WordPress version.
		$wws_wordpress_status['wp_version'] = function_exists( 'get_bloginfo' ) ? get_bloginfo( 'version' ) : __( 'N/A (get_bloginfo function does not exist)', 'woocommerce-wallet-system' );

		// Get and count active WordPress plugins.
		$wws_wordpress_status['wp_active_plugins'] = function_exists( 'get_option' ) ? count( get_option( 'active_plugins' ) ) : __( 'N/A (get_option function does not exist)', 'woocommerce-wallet-system' );

		// See if this site is multisite or not.
		$wws_wordpress_status['wp_multisite'] = function_exists( 'is_multisite' ) && is_multisite() ? __( 'Yes', 'woocommerce-wallet-system' ) : __( 'No', 'woocommerce-wallet-system' );

		// See if WP Debug is enabled.
		$wws_wordpress_status['wp_debug_enabled'] = defined( 'WP_DEBUG' ) ? __( 'Yes', 'woocommerce-wallet-system' ) : __( 'No', 'woocommerce-wallet-system' );

		// See if WP Cache is enabled.
		$wws_wordpress_status['wp_cache_enabled'] = defined( 'WP_CACHE' ) ? __( 'Yes', 'woocommerce-wallet-system' ) : __( 'No', 'woocommerce-wallet-system' );

		// Get the total number of WordPress users on the site.
		$wws_wordpress_status['wp_users'] = function_exists( 'count_users' ) ? count_users() : __( 'N/A (count_users function does not exist)', 'woocommerce-wallet-system' );

		// Get the number of published WordPress posts.
		$wws_wordpress_status['wp_posts'] = wp_count_posts()->publish >= 1 ? wp_count_posts()->publish : __( '0', 'woocommerce-wallet-system' );

		// Get PHP memory limit.
		$wws_system_status['php_memory_limit'] = function_exists( 'ini_get' ) ? (int) ini_get( 'memory_limit' ) : __( 'N/A (ini_get function does not exist)', 'woocommerce-wallet-system' );

		// Get the PHP error log path.
		$wws_system_status['php_error_log_path'] = ! ini_get( 'error_log' ) ? __( 'N/A', 'woocommerce-wallet-system' ) : ini_get( 'error_log' );

		// Get PHP max upload size.
		$wws_system_status['php_max_upload'] = function_exists( 'ini_get' ) ? (int) ini_get( 'upload_max_filesize' ) : __( 'N/A (ini_get function does not exist)', 'woocommerce-wallet-system' );

		// Get PHP max post size.
		$wws_system_status['php_max_post'] = function_exists( 'ini_get' ) ? (int) ini_get( 'post_max_size' ) : __( 'N/A (ini_get function does not exist)', 'woocommerce-wallet-system' );

		// Get the PHP architecture.
		if ( PHP_INT_SIZE == 4 ) {
			$wws_system_status['php_architecture'] = '32-bit';
		} elseif ( PHP_INT_SIZE == 8 ) {
			$wws_system_status['php_architecture'] = '64-bit';
		} else {
			$wws_system_status['php_architecture'] = 'N/A';
		}

		// Get server host name.
		$wws_system_status['server_hostname'] = function_exists( 'gethostname' ) ? gethostname() : __( 'N/A (gethostname function does not exist)', 'woocommerce-wallet-system' );

		// Show the number of processes currently running on the server.
		$wws_system_status['processes'] = function_exists( 'exec' ) ? @exec( 'ps aux | wc -l' ) : __( 'N/A (make sure exec is enabled)', 'woocommerce-wallet-system' );

		// Get the memory usage.
		$wws_system_status['memory_usage'] = function_exists( 'memory_get_peak_usage' ) ? round( memory_get_peak_usage( true ) / 1024 / 1024, 2 ) : 0;

		// Get CPU usage.
		// Check to see if system is Windows, if so then use an alternative since sys_getloadavg() won't work.
		if ( stristr( PHP_OS, 'win' ) ) {
			$wws_system_status['is_windows'] = true;
			$wws_system_status['windows_cpu_usage'] = function_exists( 'exec' ) ? @exec( 'wmic cpu get loadpercentage /all' ) : __( 'N/A (make sure exec is enabled)', 'woocommerce-wallet-system' );
		}

		// Get the memory limit.
		$wws_system_status['memory_limit'] = function_exists( 'ini_get' ) ? (int) ini_get( 'memory_limit' ) : __( 'N/A (ini_get function does not exist)', 'woocommerce-wallet-system' );

		// Get the PHP maximum execution time.
		$wws_system_status['php_max_execution_time'] = function_exists( 'ini_get' ) ? ini_get( 'max_execution_time' ) : __( 'N/A (ini_get function does not exist)', 'woocommerce-wallet-system' );

		// Get outgoing IP address.
		$wws_system_status['outgoing_ip'] = function_exists( 'file_get_contents' ) ? file_get_contents( 'http://ipecho.net/plain' ) : __( 'N/A (file_get_contents function does not exist)', 'woocommerce-wallet-system' );

		$wws_system_data['php'] = $wws_system_status;
		$wws_system_data['wp']  = $wws_wordpress_status;

		return $wws_system_data;
	}

	/**
	 * Generate html components.
	 *
	 * @param  string $wws_components    html to display.
	 * @since  1.0.0
	 */
	public function mwb_wws_plug_generate_html( $wws_components = array() ) {
		if ( is_array( $wws_components ) && ! empty( $wws_components ) ) {
			foreach ( $wws_components as $wws_component ) {
				if ( ! empty( $wws_component['type'] ) && ! empty( $wws_component['id'] ) ) {
					switch ( $wws_component['type'] ) {

						case 'hidden':
						case 'number':
						case 'email':
						case 'text':
							?>
						<div class="mwb-form-group mwb-wws-<?php echo esc_attr( $wws_component['type'] ); ?>">
							<div class="mwb-form-group__label">
								<label for="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control">
								<label class="mdc-text-field mdc-text-field--outlined">
									<span class="mdc-notched-outline">
										<span class="mdc-notched-outline__leading"></span>
										<span class="mdc-notched-outline__notch">
											<?php if ( 'number' != $wws_component['type'] ) { ?>
												<span class="mdc-floating-label" id="my-label-id" style=""><?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?></span>
											<?php } ?>
										</span>
										<span class="mdc-notched-outline__trailing"></span>
									</span>
									<input
									class="mdc-text-field__input <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>" 
									name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
									id="<?php echo esc_attr( $wws_component['id'] ); ?>"
									type="<?php echo esc_attr( $wws_component['type'] ); ?>"
									value="<?php echo ( isset( $wws_component['value'] ) ? esc_attr( $wws_component['value'] ) : '' ); ?>"
									placeholder="<?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?>"
									>
								</label>
								<div class="mdc-text-field-helper-line">
									<div class="mdc-text-field-helper-text--persistent mwb-helper-text" id="" aria-hidden="true"><?php echo ( isset( $wws_component['description'] ) ? esc_attr( $wws_component['description'] ) : '' ); ?></div>
								</div>
							</div>
						</div>
							<?php
							break;

						case 'password':
							?>
						<div class="mwb-form-group">
							<div class="mwb-form-group__label">
								<label for="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control">
								<label class="mdc-text-field mdc-text-field--outlined mdc-text-field--with-trailing-icon">
									<span class="mdc-notched-outline">
										<span class="mdc-notched-outline__leading"></span>
										<span class="mdc-notched-outline__notch">
										</span>
										<span class="mdc-notched-outline__trailing"></span>
									</span>
									<input 
									class="mdc-text-field__input <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?> mwb-form__password" 
									name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
									id="<?php echo esc_attr( $wws_component['id'] ); ?>"
									type="<?php echo esc_attr( $wws_component['type'] ); ?>"
									value="<?php echo ( isset( $wws_component['value'] ) ? esc_attr( $wws_component['value'] ) : '' ); ?>"
									placeholder="<?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?>"
									>
									<i class="material-icons mdc-text-field__icon mdc-text-field__icon--trailing mwb-password-hidden" tabindex="0" role="button">visibility</i>
								</label>
								<div class="mdc-text-field-helper-line">
									<div class="mdc-text-field-helper-text--persistent mwb-helper-text" id="" aria-hidden="true"><?php echo ( isset( $wws_component['description'] ) ? esc_attr( $wws_component['description'] ) : '' ); ?></div>
								</div>
							</div>
						</div>
							<?php
							break;

						case 'textarea':
							?>
						<div class="mwb-form-group">
							<div class="mwb-form-group__label">
								<label class="mwb-form-label" for="<?php echo esc_attr( $wws_component['id'] ); ?>"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control">
								<label class="mdc-text-field mdc-text-field--outlined mdc-text-field--textarea"  	for="text-field-hero-input">
									<span class="mdc-notched-outline">
										<span class="mdc-notched-outline__leading"></span>
										<span class="mdc-notched-outline__notch">
											<span class="mdc-floating-label"><?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?></span>
										</span>
										<span class="mdc-notched-outline__trailing"></span>
									</span>
									<span class="mdc-text-field__resizer">
										<textarea class="mdc-text-field__input <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>" rows="2" cols="25" aria-label="Label" name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>" id="<?php echo esc_attr( $wws_component['id'] ); ?>" placeholder="<?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?>"><?php echo ( isset( $wws_component['value'] ) ? esc_textarea( $wws_component['value'] ) : '' ); // WPCS: XSS ok. ?></textarea>
									</span>
								</label>

							</div>
						</div>

							<?php
							break;

						case 'select':
						case 'multiselect':
							?>
						<div class="mwb-form-group">
							<div class="mwb-form-group__label">
								<label class="mwb-form-label" for="<?php echo esc_attr( $wws_component['id'] ); ?>"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control">
								<div class="mwb-form-select">
									<select id="<?php echo esc_attr( $wws_component['id'] ); ?>" name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : '' ); ?><?php echo ( 'multiselect' === $wws_component['type'] ) ? '[]' : ''; ?>" id="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mdl-textfield__input <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>" <?php echo 'multiselect' === $wws_component['type'] ? 'multiple="multiple"' : ''; ?> >
										<?php
										foreach ( $wws_component['options'] as $wws_key => $wws_val ) {
											?>
											<option value="<?php echo esc_attr( $wws_key ); ?>"
												<?php
												if ( is_array( $wws_component['value'] ) ) {
													selected( in_array( (string) $wws_key, $wws_component['value'], true ), true );
												} else {
													selected( $wws_component['value'], (string) $wws_key );
												}
												?>
												>
												<?php echo esc_html( $wws_val ); ?>
											</option>
											<?php
										}
										?>
									</select>
									<label class="mdl-textfield__label" for="octane"><?php echo esc_html( $wws_component['description'] ); ?><?php echo ( isset( $wws_component['description'] ) ? esc_attr( $wws_component['description'] ) : '' ); ?></label>
								</div>
							</div>
						</div>

							<?php
							break;

						case 'checkbox':
							?>
						<div class="mwb-form-group">
							<div class="mwb-form-group__label">
								<label for="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control mwb-pl-4">
								<div class="mdc-form-field">
									<div class="mdc-checkbox">
										<input 
										name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
										id="<?php echo esc_attr( $wws_component['id'] ); ?>"
										type="checkbox"
										class="mdc-checkbox__native-control <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>"
										value="<?php echo ( isset( $wws_component['value'] ) ? esc_attr( $wws_component['value'] ) : '' ); ?>"
										<?php checked( $wws_component['value'], '1' ); ?>
										/>
										<div class="mdc-checkbox__background">
											<svg class="mdc-checkbox__checkmark" viewBox="0 0 24 24">
												<path class="mdc-checkbox__checkmark-path" fill="none" d="M1.73,12.91 8.1,19.28 22.79,4.59"/>
											</svg>
											<div class="mdc-checkbox__mixedmark"></div>
										</div>
										<div class="mdc-checkbox__ripple"></div>
									</div>
									<label for="checkbox-1"><?php echo ( isset( $wws_component['description'] ) ? esc_attr( $wws_component['description'] ) : '' ); ?></label>
								</div>
							</div>
						</div>
							<?php
							break;

						case 'radio':
							?>
						<div class="mwb-form-group">
							<div class="mwb-form-group__label">
								<label for="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control mwb-pl-4">
								<div class="mwb-flex-col">
									<?php
									foreach ( $wws_component['options'] as $wws_radio_key => $wws_radio_val ) {
										?>
										<div class="mdc-form-field">
											<div class="mdc-radio">
												<input
												name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
												value="<?php echo esc_attr( $wws_radio_key ); ?>"
												type="radio"
												class="mdc-radio__native-control <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>"
												<?php checked( $wws_radio_key, $wws_component['value'] ); ?>
												>
												<div class="mdc-radio__background">
													<div class="mdc-radio__outer-circle"></div>
													<div class="mdc-radio__inner-circle"></div>
												</div>
												<div class="mdc-radio__ripple"></div>
											</div>
											<label for="radio-1"><?php echo esc_html( $wws_radio_val ); ?></label>
										</div>	
										<?php
									}
									?>
								</div>
							</div>
						</div>
							<?php
							break;

						case 'radio-switch':
							?>

						<div class="mwb-form-group">
							<div class="mwb-form-group__label">
								<label for="" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
							</div>
							<div class="mwb-form-group__control">
								<div>
									<div class="mdc-switch">
										<div class="mdc-switch__track"></div>
										<div class="mdc-switch__thumb-underlay">
											<div class="mdc-switch__thumb"></div>
											<input name="
											<?php
											echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) );
											?>
											" type="checkbox" id="<?php echo esc_html( $wws_component['id'] ); ?>" value="on" class="mdc-switch__native-control <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>" role="switch" aria-checked="
											<?php
											if ( 'on' == $wws_component['value'] ) {
												echo 'true';
											} else {
												echo 'false';
											}
											?>
"
											<?php checked( $wws_component['value'], 'on' ); ?>
											>
										</div>
									</div>
								</div>
							</div>
						</div>
							<?php
							break;

						case 'button':
							?>
						<div class="mwb-form-group">
							<div class="mwb-form-group__label"></div>
							<div class="mwb-form-group__control">
								<button class="mdc-button mdc-button--raised" name= "<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
									id="<?php echo esc_attr( $wws_component['id'] ); ?>"> <span class="mdc-button__ripple"></span>
									<span class="mdc-button__label <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>"><?php echo ( isset( $wws_component['button_text'] ) ? esc_html( $wws_component['button_text'] ) : '' ); ?></span>
								</button>
							</div>
						</div>

							<?php
							break;

						case 'multi':
							?>
							<div class="mwb-form-group mwb-isfw-<?php echo esc_attr( $wws_component['type'] ); ?>">
								<div class="mwb-form-group__label">
									<label for="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
									</div>
									<div class="mwb-form-group__control">
									<?php
									foreach ( $wws_component['value'] as $component ) {
										?>
											<label class="mdc-text-field mdc-text-field--outlined">
												<span class="mdc-notched-outline">
													<span class="mdc-notched-outline__leading"></span>
													<span class="mdc-notched-outline__notch">
														<?php if ( 'number' != $component['type'] ) { ?>
															<span class="mdc-floating-label" id="my-label-id" style=""><?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?></span>
														<?php } ?>
													</span>
													<span class="mdc-notched-outline__trailing"></span>
												</span>
												<input 
												class="mdc-text-field__input <?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>" 
												name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
												id="<?php echo esc_attr( $component['id'] ); ?>"
												type="<?php echo esc_attr( $component['type'] ); ?>"
												value="<?php echo ( isset( $wws_component['value'] ) ? esc_attr( $wws_component['value'] ) : '' ); ?>"
												placeholder="<?php echo ( isset( $wws_component['placeholder'] ) ? esc_attr( $wws_component['placeholder'] ) : '' ); ?>"
												<?php echo esc_attr( ( 'number' === $component['type'] ) ? 'max=10 min=0' : '' ); ?>
												>
											</label>
								<?php } ?>
									<div class="mdc-text-field-helper-line">
										<div class="mdc-text-field-helper-text--persistent mwb-helper-text" id="" aria-hidden="true"><?php echo ( isset( $wws_component['description'] ) ? esc_attr( $wws_component['description'] ) : '' ); ?></div>
									</div>
								</div>
							</div>
								<?php
							break;
						case 'color':
						case 'date':
						case 'file':
							?>
							<div class="mwb-form-group mwb-isfw-<?php echo esc_attr( $wws_component['type'] ); ?>">
								<div class="mwb-form-group__label">
									<label for="<?php echo esc_attr( $wws_component['id'] ); ?>" class="mwb-form-label"><?php echo ( isset( $wws_component['title'] ) ? esc_html( $wws_component['title'] ) : '' ); // WPCS: XSS ok. ?></label>
								</div>
								<div class="mwb-form-group__control">
									<label class="mdc-text-field mdc-text-field--outlined">
										<input 
										class="<?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>" 
										name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
										id="<?php echo esc_attr( $wws_component['id'] ); ?>"
										type="<?php echo esc_attr( $wws_component['type'] ); ?>"
										value="<?php echo ( isset( $wws_component['value'] ) ? esc_attr( $wws_component['value'] ) : '' ); ?>"
										<?php echo esc_html( ( 'date' === $wws_component['type'] ) ? 'max=' . gmdate( 'Y-m-d', strtotime( gmdate( 'Y-m-d', mktime() ) . ' + 365 day' ) ) . 'min=' . gmdate( 'Y-m-d' ) . '' : '' ); ?>
										>
									</label>
									<div class="mdc-text-field-helper-line">
										<div class="mdc-text-field-helper-text--persistent mwb-helper-text" id="" aria-hidden="true"><?php echo ( isset( $wws_component['description'] ) ? esc_attr( $wws_component['description'] ) : '' ); ?></div>
									</div>
								</div>
							</div>
							<?php
							break;

						case 'submit':
							?>
						<tr valign="top">
							<td scope="row">
								<input type="submit" class="button button-primary" 
								name="<?php echo ( isset( $wws_component['name'] ) ? esc_html( $wws_component['name'] ) : esc_html( $wws_component['id'] ) ); ?>"
								id="<?php echo esc_attr( $wws_component['id'] ); ?>"
								class="<?php echo ( isset( $wws_component['class'] ) ? esc_attr( $wws_component['class'] ) : '' ); ?>"
								value="<?php echo esc_attr( $wws_component['button_text'] ); ?>"
								/>
							</td>
						</tr>
							<?php
							break;

						default:
							break;
					}
				}
			}
		}
	}


}
