<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://makewebbetter.com/
 * @since      1.0.0
 *
 * @package    Woocommerce_Wallet_System
 * @subpackage Woocommerce_Wallet_System/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 * namespace woocommerce_wallet_system_public.
 *
 * @package    Woocommerce_Wallet_System
 * @subpackage Woocommerce_Wallet_System/public
 * @author     WP Swings <webmaster@wpswings.com>
 */
class Woocommerce_Wallet_System_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function wws_public_enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, WOOCOMMERCE_WALLET_SYSTEM_DIR_URL . 'public/src/scss/woocommerce-wallet-system-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function wws_public_enqueue_scripts() {

		wp_register_script( $this->plugin_name, WOOCOMMERCE_WALLET_SYSTEM_DIR_URL . 'public/src/js/woocommerce-wallet-system-public.js', array( 'jquery' ), $this->version, false );
		wp_localize_script(
			$this->plugin_name,
			'wws_public_param',
			array(
				'ajaxurl'                      => admin_url( 'admin-ajax.php' ),
				'wws_ajax_error'               => __( 'An error occured!', 'woocommerce-wallet-system' ),
				'wws_amount_error'             => __( 'Enter amount greater than 0', 'woocommerce-wallet-system' ),
				'wws_transfer_amount_error'    => __( 'Transfer amount should be less than or equal to wallet balance.', 'woocommerce-wallet-system' ),
				'wws_recharge_minamount_error' => __( 'Recharge amount should be greater than or equal to ', 'woocommerce-wallet-system' ),
				'wws_recharge_maxamount_error' => __( 'Recharge amount should be less than or equal to ', 'woocommerce-wallet-system' ),
				'wws_copied_code'              => __( 'Link Copied', 'woocommerce-wallet-system' ),
			)
		);
		wp_enqueue_script( $this->plugin_name );

	}

	/**
	 * Add content to the wallet page.
	 */
	public function mwb_wws_display_wallet_endpoint_content() {
		include_once WOOCOMMERCE_WALLET_SYSTEM_DIR_PATH . 'public/partials/woocommerce-wallet-system-public-display.php';
	}

	/**
	 * Add content to wallet topup page through qrcode.
	 */
	public function mwb_wws_display_wallet_qrcode_topup_content() {
		include_once WOOCOMMERCE_WALLET_SYSTEM_DIR_PATH . 'public/partials/woocommerce-wallet-system-wallet-qrcode-topup.php';
	}

	/**
	 *  Register new endpoint to use for My Account page.
	 */
	public function mwb_wws_wallet_register_endpoint() {
		global $wp_rewrite;
		add_rewrite_endpoint( 'mwb-wallet-topup', EP_PERMALINK | EP_PAGES );
		add_rewrite_endpoint( 'mwb-qrcode-image', EP_PERMALINK | EP_PAGES );
		$wp_rewrite->flush_rules();
	}

	/**
	 *  Add new query var.
	 *
	 * @param array $vars    Query variable.
	 */
	public function mwb_wws_wallet_query_var( $vars ) {
		$vars[] = 'mwb-wallet-topup';
		$vars[] = 'mwb-qrcode-image';
		return $vars;
	}

	/**
	 * Add user id as custom data to cart item.
	 *
	 * @param array $cart_item_data cart item data.
	 * @param int   $product_id product id.
	 * @return array
	 */
	public function add_wallet_user_id_in_cart( $cart_item_data, $product_id ) {
		if ( WC()->session->__isset( 'wallet_recharge' ) ) {
			$wallet_recharge = WC()->session->get( 'wallet_recharge' );
			if ( isset( $wallet_recharge ) && ! empty( $wallet_recharge ) ) {
				$cart_item_data['user_id'] = $wallet_recharge['userid'];
				if ( array_key_exists( 'recharge_reason', $wallet_recharge ) ) {
					$cart_item_data['recharge_reason'] = $wallet_recharge['recharge_reason'];
				}
			}
		}
		return $cart_item_data;
	}

	/**
	 * Get cart item from woocommerce cart session
	 *
	 * @param array  $item cart item.
	 * @param array  $values cart item value.
	 * @param string $key cart item key.
	 * @return array
	 */
	public function mwb_wws_get_cart_items_from_session( $item, $values, $key ) {
		if ( array_key_exists( 'user_id', $values ) ) {
			$item['user_id'] = $values['user_id'];
		}
		if ( array_key_exists( 'recharge_reason', $values ) ) {
			$item['recharge_reason'] = $values['recharge_reason'];
		}
		return $item;
	}

	/**
	 * Show custom data added to cart item in cart section.
	 *
	 * @param string $product_name product name.
	 * @param array  $values cart item value.
	 * @param string $cart_item_key cart item key.
	 * @return string
	 */
	public function mwb_wws_show_user_custom_data_in_cart( $product_name, $values, $cart_item_key ) {
		if ( $values['user_id'] > 0 ) {
			$return_string  = $product_name . "</a><dl class='variation'>";
			$return_string .= "<table class='wdm_options_table' id='" . $values['product_id'] . "'>";
			$return_string .= '<tr><td>' . $values['user_id'] . $values['recharge_reason'] . '</td></tr>';
			$return_string .= '</table></dl>';
			return $return_string;
		} else {
			return $product_name;
		}
	}

	/**
	 * Add user id to order meta.
	 *
	 * @param int   $item_id order item id.
	 * @param array $values order item value.
	 * @return void
	 */
	public function mwb_wws_add_values_to_order_item_meta( $item_id, $values ) {
		$wallet_id = get_option( 'mwb_wsfw_rechargeable_product_id', '' );
		$item      = new WC_Order_Item_Product( $item_id );
		$order_id  = $item->get_order_id();
		// The product ID.
		$product_id = $item->get_product_id();
		if ( isset( $product_id ) && ! empty( $product_id ) && $product_id == $wallet_id ) {
			if ( array_key_exists( 'user_id', $values ) ) {
				$user_id = $values['user_id'];
				if ( ! empty( $user_id ) ) {
					update_post_meta( $order_id, 'mwb_wws_user_id', $user_id );
				}
			}
			if ( array_key_exists( 'recharge_reason', $values ) ) {
				$recharge_reason = $values['recharge_reason'];

				if ( ! empty( $recharge_reason ) ) {
					update_post_meta( $order_id, 'mwb_wws_recharge_reason', $recharge_reason );
				}
			}
		}

	}

	/**
	 * REmove custom data user id from cart item data.
	 *
	 * @param  string $cart_item_key cart item key.
	 * @return void
	 */
	public function mwb_wws_remove_user_id_from_cart( $cart_item_key ) {
		global $woocommerce;
		$cart = $woocommerce->cart->get_cart();
		foreach ( $cart as $key => $values ) {
			if ( $values['user_id'] == $cart_item_key ) {
				unset( $woocommerce->cart->cart_contents[ $key ] );
			}
		}
	}

	/**
	 * Check whether order has user id as meta key
	 *
	 * @param int $user_id user id.
	 * @param int $order_id order id.
	 * @return int
	 */
	public function wws_check_order_meta_for_userid( $user_id, $order_id ) {
		$wallet_userid = get_post_meta( $order_id, 'mwb_wws_user_id', true );
		return $wallet_userid;
	}

	/**
	 * Check whether order has recharge_reason as meta key
	 *
	 * @param string $recharge_reason reason for recharge.
	 * @param int    $order_id order id.
	 * @return string
	 */
	public function wws_check_order_meta_for_recharge_reason( $recharge_reason = '', $order_id ) {
		$recharge_reason = get_post_meta( $order_id, 'mwb_wws_recharge_reason', true );
		return $recharge_reason;
	}

	/**
	 * Show minimum and maximum value for wallet recharge.
	 *
	 * @return array
	 */
	public function min_max_value_for_wallet_recharge() {
		$min_wallet_recharge_amount = get_option( 'wws_min_wallet_recharge_amount', 0 );
		$max_wallet_recharge_amount = get_option( 'wws_max_wallet_recharge_amount', '' );
		$wallet_recharge_amount              = array();
		$wallet_recharge_amount['min_value'] = $min_wallet_recharge_amount;
		$wallet_recharge_amount['max_value'] = $max_wallet_recharge_amount;
		return $wallet_recharge_amount;
	}

	/**
	 * Remove custom data added to cart item during removal of cart item.
	 *
	 * @param  string $removed_cart_item_key removed cart item key.
	 * @return void
	 */
	public function mwb_wss_remove_value_from_session( $removed_cart_item_key ) {
		$cart = WC()->cart->get_cart();
		foreach ( $cart as $key => $values ) {
			if ( array_key_exists( 'user_id', $values ) ) {
				if ( $values['user_id'] == $removed_cart_item_key ) {
					unset( WC()->cart->cart_contents[ $key ] );
				}
			}
			if ( array_key_exists( 'recharge_reason', $values ) ) {
				if ( $values['recharge_reason'] == $removed_cart_item_key ) {
					unset( WC()->cart->cart_contents[ $key ] );
				}
			}
		}
	}

	/**
	 * Show withdrawal message in widthdrawal request form.
	 *
	 * @return string
	 */
	public function mwb_wws_show_withdrawal_message() {
		$show_withdrawal_message = get_option( 'wws_withdrawal_page_message', '' );
		return $show_withdrawal_message;
	}

	/**
	 * Add invitation link if user does not exist.
	 *
	 * @return string
	 */
	public function mwb_wws_add_invitation_link_message() {
		$invitation_link = '<a href="" id="mwb_wws_open_invitation_box" >' . esc_html__( 'Invite him/her to get the wallet amount', 'woocommerce-wallet-system' ) . '</a>';
		return $invitation_link;
	}

	/**
	 * Show message after invitation link is send to user.
	 *
	 * @param string $wpg_message inviatation message.
	 * @param string $type message type.
	 * @return void
	 */
	public function show_message_on_mail_send( $wpg_message, $type = 'error' ) {
		$wpg_notice = "<div class='woocommerce'><p class='" . esc_attr( $type ) . "'>" . $wpg_message . '</p></div>';
		echo wp_kses_post( $wpg_notice );
	}

	/**
	 * Add popup box in wallet transfer page for sending inviation link to user.
	 *
	 * @param string $content default content.
	 * @param int    $user_id user id.
	 * @param string $email user email id.
	 * @param int    $transfer_amount transfer amount.
	 * @return void
	 */
	public function mwb_wws_show_additional_content( $content, $user_id, $email, $transfer_amount ) {
		?>
			<div class="mwb_wallet-edit--popupwrap" id="mwb_wws_show_inviatation_link_form" >
				<div class="mwb_wallet-edit-popup">
					<p><span id="close_wallet_form"><img src="<?php echo esc_url( WALLET_SYSTEM_FOR_WOOCOMMERCE_DIR_URL ); ?>admin/image/cancel.svg"></span></p>
					<form method="post">
						<div class="mwb_wallet-edit-popup-content">
							<div class="mwb_wallet-edit-popup-amount">
								<div class="mwb_wallet-edit-popup-control">
									<label for="mwb_wallet_transfer_note"><?php esc_html_e( 'To(Name)', 'woocommerce-wallet-system' ); ?></label>
									<input type="text" id="mwb_wallet_username"  name="mwb_wallet_username" required >
								</div>
							</div>
						</div>
						<div class="mwb_wallet-edit-popup-content">
							<div class="mwb_wallet-edit-popup-amount">
								<div class="mwb_wallet-edit-popup-control">
									<label for="mwb_wallet_transfer_note"><?php esc_html_e( 'Your Message', 'woocommerce-wallet-system' ); ?></label>
									<textarea class="mwb_wallet-textarea" name="mwb_wallet_transfer_note"></textarea>
								</div>
							</div>
						</div>
						<div class="mwb_wallet-edit-popup-btn">
							<input type="hidden" id="wallet_transfer_user_id" name="wallet_transfer_user_id" value="<?php echo esc_attr( $user_id ); ?>" />
							<input type="hidden" id="mwb_wallet_user_email" name="mwb_wallet_user_email" value="<?php echo esc_attr( $email ); ?>" />
							<input type="hidden" id="mwb_transfer_amount" name="mwb_transfer_amount" value="<?php echo esc_attr( $transfer_amount ); ?>" />
							<input type="submit" name="send_invitation" class="mwb-btn mwb-btn-popup mwb-btn__filled" value="<?php esc_attr_e( 'Send Invitation', 'woocommerce-wallet-system' ); ?>">
						</div>
					</form>
				</div>
			</div>
		<?php
	}

	/**
	 * Include qrcode images template to show qr code image.
	 *
	 * @param string $template template path.
	 * @return string
	 */
	public function mwb_wws_show_qrcode_image( $template ) {
		$wallet_page_id = get_option( 'mwb_wws_wallet_top_page_id' );
		if ( ! empty( $wallet_page_id ) ) {
			if ( is_page( $wallet_page_id ) && ( get_query_var( 'mwb-qrcode-image', false ) !== false ) ) {
				$newTemplate = WOOCOMMERCE_WALLET_SYSTEM_DIR_PATH . 'public/partials/woocommerce-wallet-system-wallet-qrcode-image.php';
				if ( file_exists( $newTemplate ) ) {
					return $newTemplate;
				}
			}
		}
		return $template;

	}

	/**
	 * Includes topup page to transfer amount or recharge wallet created through qr code.
	 *
	 * @param string $content page content.
	 * @return string
	 */
	public function content_for_my_wallet_page_wallet_transfer( $content ) {
		$wallet_page_id = get_option( 'mwb_wws_wallet_top_page_id' );
		if ( ! empty( $wallet_page_id ) ) {
			if ( is_page( $wallet_page_id ) && ( get_query_var( 'mwb-wallet-topup', false ) !== false ) ) {
				include_once WOOCOMMERCE_WALLET_SYSTEM_DIR_PATH . 'public/partials/woocommerce-wallet-system-wallet-qrcode-topup.php';
			}
		}
		return $content;
	}

	/**
	 * Send mail to user for registration.
	 *
	 * @param string $to user email address.
	 * @param string $subject subject for mail.
	 * @param string $mail_message message for mail.
	 * @param string $headers data to be send in header.
	 * @return boolean
	 */
	public function send_mail_to_user_for_registration( $to, $subject, $mail_message, $headers ) {
		// Here put your Validation and send mail.
		$send_mail = wp_mail( $to, $subject, $mail_message, $headers );
		return $send_mail;
	}

	/**
	 * Add wallet transferred amount to wallet after user registration.
	 *
	 * @param int $user_id user id.
	 * @return void
	 */
	public function mwb_save_wallet_amount_after_registration( $user_id ) {
		if ( ! empty( $_GET['transfer_amount'] ) && ! empty( $_GET['user_id'] ) ) {
			$currency        = get_woocommerce_currency();
			$transfer_amount = sanitize_text_field( wp_unslash( $_GET['transfer_amount'] ) );
			$another_user_id = sanitize_text_field( wp_unslash( $_GET['user_id'] ) );
			$wallet_bal      = get_user_meta( $another_user_id, 'mwb_wallet', true );
			$returnid        = update_user_meta( $user_id, 'mwb_wallet', $transfer_amount );
			$transfer_note   = ! empty( $_GET['transfer_note'] ) ? sanitize_text_field( wp_unslash( $_GET['transfer_note'] ) ) : '';
			if ( $returnid ) {
				$wallet_payment_gateway = new Wallet_System_For_Woocommerce();
				$send_email_enable      = get_option( 'mwb_wsfw_enable_email_notification_for_wallet_update', '' );
				// first user.
				$user1 = get_user_by( 'id', $user_id );
				$name1 = $user1->first_name . ' ' . $user1->last_name;

				$user2 = get_user_by( 'id', $another_user_id );
				$name2 = $user2->first_name . ' ' . $user2->last_name;
				if ( isset( $send_email_enable ) && 'on' === $send_email_enable ) {

					$mail_text1  = esc_html__( 'Hello ', 'woocommerce-wallet-system' ) . esc_html( $name1 ) . __( ',<br/>', 'woocommerce-wallet-system' );
					$mail_text1 .= __( 'Wallet credited by ', 'woocommerce-wallet-system' ) . wc_price( $transfer_amount, array( 'currency' => $currency ) ) . __( ' through wallet transfer by ', 'woocommerce-wallet-system' ) . $name2;
					$to1         = $user1->user_email;
					$from        = get_option( 'admin_email' );
					$subject     = __( 'Wallet updating notification', 'woocommerce-wallet-system' );
					$headers1    = 'MIME-Version: 1.0' . "\r\n";
					$headers1   .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
					$headers1   .= 'From: ' . $from . "\r\n" .
						'Reply-To: ' . $to1 . "\r\n";

					$wallet_payment_gateway->send_mail_on_wallet_updation( $to1, $subject, $mail_text1, $headers1 );

				}

				$transaction_type     = __( 'Wallet credited by user ', 'woocommerce-wallet-system' ) . $user2->user_email . __( ' to user ', 'woocommerce-wallet-system' ) . $user1->user_email;
				$wallet_transfer_data = array(
					'user_id'          => $user_id,
					'amount'           => $transfer_amount,
					'currency'         => $currency,
					'payment_method'   => __( 'Wallet Transfer', 'woocommerce-wallet-system' ),
					'transaction_type' => $transaction_type,
					'order_id'         => '',
					'note'             => $transfer_note,

				);

				$wallet_payment_gateway->insert_transaction_data_in_table( $wallet_transfer_data );

				$wallet_bal -= $transfer_amount;
				$update_user = update_user_meta( $another_user_id, 'mwb_wallet', abs( $wallet_bal ) );
				if ( $update_user ) {

					if ( isset( $send_email_enable ) && 'on' === $send_email_enable ) {
						$mail_text2  = esc_html__( 'Hello ', 'woocommerce-wallet-system' ) . esc_html( $name2 ) . __( ',<br/>', 'woocommerce-wallet-system' );
						$mail_text2 .= __( 'Wallet debited by ', 'woocommerce-wallet-system' ) . wc_price( $transfer_amount, array( 'currency' => $currency ) ) . __( ' through wallet transfer to ', 'woocommerce-wallet-system' ) . $name1;
						$to2         = $user2->user_email;
						$headers2    = 'MIME-Version: 1.0' . "\r\n";
						$headers2   .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
						$headers2   .= 'From: ' . $from . "\r\n" .
							'Reply-To: ' . $to2 . "\r\n";

						$wallet_payment_gateway->send_mail_on_wallet_updation( $to2, $subject, $mail_text2, $headers2 );
					}
					$transaction_type = __( 'Wallet debited from user ', 'woocommerce-wallet-system' ) . $user2->user_email . __( ' wallet, transferred to user ', 'woocommerce-wallet-system' ) . $user1->user_email;
					$transaction_data = array(
						'user_id'          => $another_user_id,
						'amount'           => $transfer_amount,
						'currency'         => $currency,
						'payment_method'   => __( 'Wallet Transfer', 'woocommerce-wallet-system' ),
						'transaction_type' => $transaction_type,
						'order_id'         => '',
						'note'             => $transfer_note,

					);

					$wallet_payment_gateway->insert_transaction_data_in_table( $transaction_data );

				}
			}
		}
	}

	/**
	 * Show message on form submit
	 *
	 * @param string $wpg_message message to be shown on form submission.
	 * @param string $type error type.
	 * @return void
	 */
	public function show_message_on_form_submit( $wpg_message, $type = 'error' ) {
		$wpg_notice = '<div class="woocommerce"><p class="' . esc_attr( $type ) . '">' . $wpg_message . '</p>	</div>';
		echo wp_kses_post( $wpg_notice );
	}

	/**
	 * On woocommerce init starts woocmmerce session.
	 *
	 * @return void
	 */
	public function mwb_wws_set_woocoomerce_session() {

		if ( ! empty( WC()->session ) && ! WC()->session->has_session() ) {
			WC()->session->set_customer_session_cookie( true );
		}
	}

	/**
	 * Return QR Code from a string
	 *
	 * @param string $str qr code path.
	 * @param string $starting_word starting word.
	 * @param string $ending_word ending word.
	 * @return string
	 */
	public function string_between_two_string( $str, $starting_word, $ending_word ) {
		$arr = explode( $starting_word, $str );
		if ( isset( $arr[1] ) ) {
			$arr = explode( $ending_word, $arr[1] );
			return $arr[0];
		}
		return '';
	}

	/**
	 * Returns converted price of wallet balance.
	 *
	 * @param float $wallet_bal wallet balance.
	 * @return float
	 */
	public function mwb_wws_show_converted_price( $wallet_bal ) {
		if ( function_exists( 'mwb_mmcsfw_get_currenct_currency' ) ) {
			$current_currency = mwb_mmcsfw_get_currenct_currency();
			if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency' ) ) {
				$amount = mwb_mmcsfw_admin_fetch_currency_rates_from_base_currency( $current_currency, $wallet_bal );
				return $amount;

			}
		} else {
			return $wallet_bal;
		}
	}

	/**
	 * Returns converted amount for recharge.
	 *
	 * @param float $recharge_amount recharge amount.
	 * @return float
	 */
	public function mwb_wws_convert_price_on_cart( $recharge_amount ) {
		if ( function_exists( 'mwb_mmcsfw_get_currenct_currency' ) ) {
			$current_currency = mwb_mmcsfw_get_currenct_currency();
			if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_to_base_currency' ) ) {
				$base_amount               = mwb_mmcsfw_admin_fetch_currency_rates_to_base_currency( $current_currency, $recharge_amount );
				$converted_recharge_amount = apply_filters( 'mwb_wsfw_show_converted_price', $base_amount );
				return $converted_recharge_amount;

			}
		} else {
			return $recharge_amount;
		}
	}

	/**
	 * Convert the amount into base currency amount.
	 *
	 * @param string $price price.
	 * @return string
	 */
	public function mwb_wws_convert_to_base_price( $price ) {
		if ( function_exists( 'mwb_mmcsfw_get_currenct_currency' ) ) {
			$current_currency = mwb_mmcsfw_get_currenct_currency();
			if ( function_exists( 'mwb_mmcsfw_admin_fetch_currency_rates_to_base_currency' ) ) {
				$base_amount = mwb_mmcsfw_admin_fetch_currency_rates_to_base_currency( $current_currency, $price );
				return $base_amount;

			}
		} else {
			return $price;
		}
	}

	/**
	 * Returns the current currency.
	 *
	 * @param string $currency currency.
	 * @return string
	 */
	public function mwb_wws_get_current_currency( $currency ) {
		if ( function_exists( 'mwb_mmcsfw_get_currenct_currency' ) ) {
			$current_currency = mwb_mmcsfw_get_currenct_currency();
			return $current_currency;
		} else {
			return $currency;
		}
	}

	/**
	 * Send invitation link to user during wallet transfer.
	 *
	 * @return void
	 */
	public function mwb_wws_send_mail_to_user() {
		if ( isset( $_POST['send_invitation'] ) && ! empty( $_POST['send_invitation'] ) ) {
			unset( $_POST['send_invitation'] );
			if ( ! empty( $_POST['wallet_transfer_user_id'] ) ) {
				$user_id = sanitize_text_field( wp_unslash( $_POST['wallet_transfer_user_id'] ) );
			}
			$username        = ! empty( $_POST['mwb_wallet_username'] ) ? sanitize_text_field( wp_unslash( $_POST['mwb_wallet_username'] ) ) : '';
			$useremail       = ! empty( $_POST['mwb_wallet_user_email'] ) ? sanitize_text_field( wp_unslash( $_POST['mwb_wallet_user_email'] ) ) : '';
			$transfer_note   = ! empty( $_POST['mwb_wallet_transfer_note'] ) ? sanitize_text_field( wp_unslash( $_POST['mwb_wallet_transfer_note'] ) ) : '';
			$transfer_amount = ! empty( $_POST['mwb_transfer_amount'] ) ? sanitize_text_field( wp_unslash( $_POST['mwb_transfer_amount'] ) ) : '';

			$registration_url = add_query_arg(
				array(
					'transfer_amount' => $transfer_amount,
					'user_id'         => $user_id,
					'transfer_note'   => $transfer_note,
				),
				wc_get_page_permalink( 'myaccount' )
			);
			$mail_text        = __( 'Hello, ', 'woocommerce-wallet-system' ) . $username . '<br/>';
			$mail_text       .= __( 'Click the link for registering your account on ', 'woocommerce-wallet-system' ) . get_bloginfo( 'name' ) . __( ' and you will get some amount as wallet balance.<br/>', 'woocommerce-wallet-system' ) . __( '<a href="' . esc_url( $registration_url ) . '" style="background:#2196f3;margin-top:15px;text-decoration:none;padding:6px 10px;color:#fff;display:inline-block;" >Create Account</a>', 'woocommerce-wallet-system' );
			$to               = $useremail;
			$from             = get_option( 'admin_email' );
			$subject          = 'Invitations to join the Wallet System';
			$headers          = 'MIME-Version: 1.0' . "\r\n";
			$headers          = 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
			$headers         .= 'From: ' . $from . "\r\n" .
				'Reply-To: ' . $to . "\r\n";

			$send_mail = $this->send_mail_to_user_for_registration( $to, $subject, $mail_text, $headers );
			if ( $send_mail ) {
				$this->show_message_on_mail_send( esc_html__( 'Registration mail is send to ', 'woocommerce-wallet-system' ) . $useremail . esc_html__( ' successfully', 'woocommerce-wallet-system' ), 'woocommerce-message' );
			} else {
				$this->show_message_on_mail_send( esc_html__( 'Registration mail is not send to ', 'woocommerce-wallet-system' ) . $useremail, 'woocommerce-error' );
			}
		}
	}

}
